package org.openxma.dsl.core.scoping;

import java.util.Set;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.xtext.naming.QualifiedName;
import org.eclipse.xtext.scoping.impl.ImportedNamespaceAwareLocalScopeProvider;
import org.openxma.dsl.core.model.Model;

import com.google.common.collect.Sets;

public class OpenXmaNamespaceLocalScopeProvider extends ImportedNamespaceAwareLocalScopeProvider {

    protected ImportNormalizer createImportNormalizer(final String name) {
        return new ImportNormalizer(new QualifiedName(name.lastIndexOf("^") == -1 ? name : name.replaceAll("\\^", "")));
    }

    protected Set<ImportNormalizer> getImportNormalizer(final EObject context) {
        Set<ImportNormalizer> importNormalizers = Sets.newLinkedHashSet();
        if (context instanceof Model) {
            Model model = (Model) context;
            importNormalizers.add(new ImportNormalizer(new QualifiedName("org.openxma.lang.StringValidator")));
            importNormalizers.add(new ImportNormalizer(new QualifiedName("org.openxma.lang.CustomValidator")));
            importNormalizers.add(new ImportNormalizer(new QualifiedName("org.openxma.lang.NumberValidator")));
            importNormalizers.add(new ImportNormalizer(new QualifiedName("org.openxma.lang.BooleanValidator")));
            importNormalizers.add(new ImportNormalizer(new QualifiedName("org.openxma.lang.DateValidator")));
            importNormalizers.add(new ImportNormalizer(new QualifiedName("org.openxma.lang.TimeStampValidator")));
            importNormalizers.add(new ImportNormalizer(new QualifiedName("org.openxma.lang.I18NEnumerationValidator")));
            importNormalizers.add(new ImportNormalizer(new QualifiedName("org.openxma.lang.BigDecimal")));
            importNormalizers.add(new ImportNormalizer(new QualifiedName("org.openxma.lang.Date")));
            importNormalizers.add(new ImportNormalizer(new QualifiedName("org.openxma.lang.Timestamp")));
            importNormalizers.add(new ImportNormalizer(new QualifiedName("org.openxma.lang.Double")));
            importNormalizers.add(new ImportNormalizer(new QualifiedName("org.openxma.lang.Float")));
            importNormalizers.add(new ImportNormalizer(new QualifiedName("org.openxma.lang.Integer")));
            importNormalizers.add(new ImportNormalizer(new QualifiedName("org.openxma.lang.Long")));
            importNormalizers.add(new ImportNormalizer(new QualifiedName("org.openxma.lang.String")));
            importNormalizers.add(new ImportNormalizer(new QualifiedName("org.openxma.lang.I18NEnumeration")));
            importNormalizers.add(new ImportNormalizer(new QualifiedName("org.openxma.lang.Boolean")));
            importNormalizers.add(new ImportNormalizer(new QualifiedName("org.openxma.lang.Blob")));
            importNormalizers.add(new ImportNormalizer(new QualifiedName("org.openxma.lang.Clob")));
            importNormalizers.add(new ImportNormalizer(new QualifiedName("org.openxma.lang.ValueObjectByteArray")));
            importNormalizers.add(new ImportNormalizer(new QualifiedName("org.openxma.lang.ValueObjectXml")));
            importNormalizers.add(new ImportNormalizer(new QualifiedName("org.openxma.styles.dataContainer")));
            importNormalizers.add(new ImportNormalizer(new QualifiedName("org.openxma.styles.actionButton")));
            importNormalizers.add(new ImportNormalizer(new QualifiedName("org.openxma.styles.variableFieldWidth")));
            importNormalizers.add(new ImportNormalizer(new QualifiedName("org.openxma.styles.useLeftTab")));
            importNormalizers.add(new ImportNormalizer(new QualifiedName("org.openxma.styles.useRightTab")));

            importNormalizers.addAll(super.getImportNormalizer(context));

            if (model.getName() != null) {
                importNormalizers.add(new ImportNormalizer(new QualifiedName(model.getName() + ".*")));
            }
        } else {
            importNormalizers = super.getImportNormalizer(context);
        }
        return importNormalizers;
    }
}
